#!/usr/bin/env python3
"""
Display security details: holding accounts, quantity, value, last price w/ date, asset class.
Calculate:
    - Return of Capital
    - Yield
argparse can effectively replace click for building CLIs.
"""
import argparse
import logging

#from piecash import Commodity, Account
from pricedb import PriceDbApplication, SecuritySymbol
from gnucash_portfolio import BookAggregate
from gnucash_portfolio.reports.security_info import SecurityInfoReport
from gnucash_portfolio.model.stock_model import SecurityDetailsViewModel


def read_parameters():
    """ Read parameters from the command line """
    parser = argparse.ArgumentParser(description='read symbol from command line')
    parser.add_argument('symbol', type=str, help='security symbol')
    parser.add_argument("-v", "--verbose", help="increase output verbosity", action="store_true")
    args = parser.parse_args()

    if args.verbose:
        logging.basicConfig(level=logging.DEBUG)

    return args


def display(model: SecurityDetailsViewModel):
    """ Format and display the results """
    # header
    #print("    security            quantity  ")
    #print("-------------------------------------------------------")
    print(f"{model.security.fullname}")

    #shares = agg.get_num_shares()

    print(f"{model.security.namespace}:{model.security.mnemonic}, shares: {model.quantity:,.2f}")

    # todo add all the info from the security details page in web ui,
    # prices, etc.
    # avg_price = agg.get_avg_price()
    #currency = agg.get_currency()
    currency = model.currency
    print(f"Average price: {model.average_price:.2f} {currency}")

    # last price
    prices_app = PriceDbApplication()
    sec_symbol = SecuritySymbol("", "")
    sec_symbol.parse(model.symbol)
    latest_price = prices_app.get_latest_price(sec_symbol)
    latest_price_date = latest_price.datum.to_iso_date_string()
    logging.debug(latest_price)
    print(f"Latest price: {latest_price.value:.2f} {latest_price.currency} on {latest_price_date}")

    print("")

    # Income
    print("Income")
    print(f"Total: {model.income:,.2f} {model.currency}, {model.income_perc:.2f}%")
    print(f"Last 12m: {model.income_last_12m:,.2f} {model.currency}, {model.income_perc_last_12m:.2f}%")

    # Return of Capital.
    if model.return_of_capital:
        print(f"Return of capital: {model.return_of_capital:,.2f}")

    print("")

    print("Holding Accounts:")
    print("-----------------")

    for account in model.accounts:
        balance = account.get_balance()
        if balance == 0:
            # Skip empty accounts
            continue
        value = balance * latest_price.value
        print(f"{account.fullname}, {balance:,.2f} units, {value:,.2f} {latest_price.currency}")


def run(args):
    """ Call this method if running from another module which parses arguments """
    symbol = args.symbol
    symbol = symbol.upper()

    book = BookAggregate()

    agg = book.securities.get_aggregate_for_symbol(symbol)
    security = agg.security

    report = SecurityInfoReport(book)
    model = report.run(symbol)

    # Display
    if security is None:
        print(f"No securities found for {symbol}.")
        exit

    display(model)


def main():
    args = read_parameters()

    run(args)

if __name__ == "__main__":
    main()



""" Test security info """

import sys
from gnucash_portfolio_cli import security_info

sys.path.insert(0, '../src')


def test_latest_price():
    """ Check the latest price date """
    #security_info()
    pass

def test_return_of_capital():
    """ debug through the return of capital calculation """
    security_info.main()
    


"""
Australian tax report
Inputs: 
- income accounts (i.e. for dividends)
- tax accounts (i.e. for withholding tax)
- the official exchange rates
- base currency

Outputs:
- all taxable income, in base currency (AUD)
- all external income, converted into the base currency
"""

# Read the configuration from the config.py file?


#!/usr/bin/env python3
"""
Trying to put the whole CLI in here.
It will allow listing all the available commands with options and parameters (if any).
The commands will invoke their respective modules.

Example: `gp-cli test me -w yo -y`
"""
import argparse


def setup_portfolio_value_command(subparsers):
    """ Set up the parser for the portfolio value command """
    from gnucash_portfolio_cli import portfolio_value

    subparser: argparse.ArgumentParser = subparsers.add_parser("pvalue")
    subparser.set_defaults(func=portfolio_value.main)

def setup_parser():
    """ Set up all the commands with parameters and arguments """
    from gnucash_portfolio_cli import (security_info, upcoming_scheduled_transactions,
        favourite_accounts)

    parser: argparse.ArgumentParser = argparse.ArgumentParser()
    parser.add_argument('--version', action='version', version='0.0.1')
    parser.set_defaults()

    subparsers = parser.add_subparsers()
    parser_test: argparse.ArgumentParser = subparsers.add_parser("test")

    # argument (required)
    parser_test.add_argument("what", type=str, help="test what?")
    # option
    parser_test.add_argument("-w", "--why", default="because!")
    # flag
    parser_test.add_argument("-y", "--yes", action="store_true")
    parser_test.set_defaults(func=show)

    # Security Info
    parser_secinfo: argparse.ArgumentParser = subparsers.add_parser("secinfo")
    parser_secinfo.add_argument("symbol", type=str, help="Security symbol to provide the information for")
    parser_secinfo.set_defaults(func=security_info.run)

    # Scheduled Transactions
    parser_scheduled: argparse.ArgumentParser = subparsers.add_parser("sched")
    parser_scheduled.set_defaults(func=upcoming_scheduled_transactions.main)

    setup_portfolio_value_command(subparsers)

    # Favourite Accounts
    fav_accounts: argparse.ArgumentParser = subparsers.add_parser("fav")
    fav_accounts.set_defaults(func=favourite_accounts.main)

    return parser

def show(args):
    print(f"yo args: what={args.what}, why={args.why}, yes={args.yes}")

def main(params = None):
    """ this code is in a function so that it can be declared an entry point in setup.py """
    import sys

    parser = setup_parser()

    args = parser.parse_args(params)

    # Handle empty arguments from the console.
    if len(sys.argv) == 1:
        params = ["-h"]
        # For debugging:
        #params = parser.parse_args(["secinfo", "vym"])
        #params = parser.parse_args(["test", "me"])
        #params = ["scheduled"]
        args = parser.parse_args(params)

    args.func(args)


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
""" search for an account """
import argparse
from gnucash_portfolio import BookAggregate

def read_parameters():
    """ Read parameters from the command line """
    parser = argparse.ArgumentParser(description='read input from command line')

    parser.add_argument('search_term', type=str, help='The term to search for in account name or description')

    args = parser.parse_args()
    return args

def main():
    """ Main entry point. Useful for unit testing """
    # Get parameter from command line
    args = read_parameters()

    book = BookAggregate()
    accounts = book.accounts.find_by_name(args.search_term)
    accounts = sorted(accounts, key=lambda account: account.fullname)

    for account in accounts:
        print(f"{account.fullname} {account.description}, {account.guid}")

if __name__ == "__main__":
    main()



#!/usr/bin/env python3
""" List your favorite accounts """

from pydatum import Datum
from gnucash_portfolio import BookAggregate

def main(args = None):
    """ Show the list of favorite accounts """
    print("Favourite accounts:\n")
    book = BookAggregate()
    favourites = book.accounts.get_favourite_account_aggregates()
    favourites = sorted(favourites, key=lambda aggregate: aggregate.account.name)

    for aggregate in favourites:
        #balance = aggregate.get_balance()
        today = Datum()
        today.today()

        balance = aggregate.get_balance_on(today.value)
        currency = aggregate.account.commodity.mnemonic

        print(f"{aggregate.account.name:<25}{balance:>10,.2f} {currency}")

    #print("\n")

if __name__ == "__main__":
    main()
